{% load aa_forum %}

<link
    rel="stylesheet"
    href="{% aa_forum_static 'aa_forum/css/aa-forum.min.css' %}"
    integrity="sha512-P1xS55LVjHNap3aJEMN1wxxpBJfZZK0Y0XVutjReE3J2/5eUc5jSY/pCZKlpmSzJUZHAU6UQrbgebLkYPs0Q/w=="
    crossorigin="anonymous"
>
